"use strict";

exports.__esModule = true;
exports.setUpdateFn = setUpdateFn;
exports.setBatchUpdatesFn = setBatchUpdatesFn;
exports.notifyManager = void 0;

var _utils = require("./utils");

// GETTERS AND SETTERS
// Default to a dummy "update" implementation that just runs the callback
var updateFn = function updateFn(callback) {
  callback();
}; // Default to a dummy "batch update" implementation that just runs the callback


var batchUpdatesFn = function batchUpdatesFn(callback) {
  callback();
};
/**
 * Use this function to set a custom update function.
 * This can be used to for example wrap updates with `React.act` while running tests.
 */


function setUpdateFn(fn) {
  updateFn = fn;
}
/**
 * Use this function to set a custom batch function to batch updates together into a single render pass.
 * By default React Query will use the batch function provided by ReactDOM or React Native.
 */


function setBatchUpdatesFn(fn) {
  batchUpdatesFn = fn;
} // CLASS


var NotifyManager = /*#__PURE__*/function () {
  function NotifyManager() {
    this.queue = [];
    this.transactions = 0;
  }

  var _proto = NotifyManager.prototype;

  _proto.batch = function batch(callback) {
    this.transactions++;
    var result = callback();
    this.transactions--;

    if (!this.transactions) {
      this.flush();
    }

    return result;
  };

  _proto.schedule = function schedule(notify) {
    if (this.transactions) {
      this.queue.push(notify);
    } else {
      (0, _utils.scheduleMicrotask)(function () {
        updateFn(notify);
      });
    }
  };

  _proto.flush = function flush() {
    var queue = this.queue;
    this.queue = [];

    if (queue.length) {
      (0, _utils.scheduleMicrotask)(function () {
        batchUpdatesFn(function () {
          queue.forEach(function (notify) {
            updateFn(notify);
          });
        });
      });
    }
  };

  return NotifyManager;
}(); // SINGLETON


var notifyManager = new NotifyManager();
exports.notifyManager = notifyManager;